/* -*- linux-c -*-
 *
 * pseudo keyboard driver
 *
 * 2000 by JR0BAK = Kunihiko IMAI <bak@d2.dion.ne.jp>
 *
 */
#define __KERNEL__
#define MODULE
#define PSEUDOKBD_MINOR		241

#include <linux/config.h>
#include <linux/module.h>

#include <asm/spinlock.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/tty.h>
#include <linux/mm.h>
#include <linux/signal.h>
#include <linux/init.h>
#include <linux/kbd_ll.h>
#include <linux/delay.h>
#include <linux/random.h>
#include <linux/poll.h>
#include <linux/miscdevice.h>
#include <linux/malloc.h>

#include <asm/keyboard.h>
#include <asm/bitops.h>
#include <asm/uaccess.h>
#include <asm/irq.h>
#include <asm/system.h>


static ssize_t pseudokbd_write (struct file * file, const char * buf,
				size_t count, loff_t *ppos)
{
	char *bp, *bufk = kmalloc(sizeof(char)*count, GFP_KERNEL );
	int i;

	if ( bufk == NULL ) return 0;
	bp = bufk;

	copy_from_user ( bufk, buf, count );
	for ( i = count; i > 0; --i ) {
		handle_scancode ( *bp, !(*bp & 0x80) );
		++bp;
	}
	kfree ( bufk );
	return count;
}


static int pseudokbd_open (struct inode * inode, struct file * file)
{
	MOD_INC_USE_COUNT;
	return 0;
}

static int pseudokbd_release (struct inode * inode, struct file * file)
{
	MOD_DEC_USE_COUNT;
	return 0;
}

static struct file_operations pseudokbd_fops = {
	open:		pseudokbd_open,
	release:	pseudokbd_release,
	write:		pseudokbd_write,
};

static struct miscdevice pseudokbd_misc = {
	PSEUDOKBD_MINOR, "pseudo keyboard", &pseudokbd_fops,
};

int pseudokbd_init(void)
{
	misc_register (&pseudokbd_misc);
	return 0;
}

int pseudokbd_cleanup(void)
{
	misc_deregister (&pseudokbd_misc);
	return 0;
}


#ifdef MODULE
int init_module(void)
{
	return pseudokbd_init();
}


void cleanup_module(void)
{
	pseudokbd_cleanup();
}
#endif /* MODULE */
